/*
 * GameEnvironment application
 * BME MIT, IRF course
 * https://www.inf.mit.bme.hu/edu/bsc/irf
 */
package hu.mit.bme.ftsrg.gameEnvironment;

import java.io.IOException;
import java.net.Socket;
import java.util.StringTokenizer;

import hu.mit.bme.ftsrg.gameEnvironment.battleShip.BattleshipGame;
import hu.mit.bme.ftsrg.gameEnvironment.battleShip.BattleshipUserInterface;
import hu.mit.bme.ftsrg.gameEnvironment.mineSweeper.MineSweeper;

/**
 * Represents a connected player to the GameEnvironment.
 * 
 * @author Istvan Szloboda
 */
public class GameSession implements Runnable {

	private Socket socket;

	public GameSession(Socket socket) throws IOException {
		this.socket = socket;
	}

	public void run() {
		try {

			System.out.println("Connection established with "
					+ socket.getInetAddress() + ":" + socket.getPort());

			SocketCommunication socketComm = new SocketCommunication(socket);

			boolean onGame = true;
			String command;

			while (onGame) {
				// Write out the main menu:
				socketComm.writeLine("Choose a game!");
				socketComm.writeLine("Battleship: \"b\"");
				socketComm.writeLine("Minesweeper \"m\"\n");
				socketComm.writeLine("Exit: \"q\" ");
				command = socketComm.readLine();
				if ("b".equals(command)) {

					// Create a Battleship game with a user-interface and launch
					// the game
					BattleshipGame btlsh = new BattleshipGame(socketComm);
					BattleshipUserInterface userInterface = new BattleshipUserInterface();
					userInterface.run(btlsh, socketComm);

				}
				if ("m".equals(command)) {

					// Ask for parameters and create a Minesweeper game
					socketComm
							.writeLine("Give the size and the count of mines <width> <height> <mines>! (Blank means: 10 10 10)");
					String initParam = socketComm.readLine();
					int width, height, mines;

					if (initParam.matches("[0-9]+ [0-9]+ [0-9]+")) {
						StringTokenizer tok = new StringTokenizer(initParam);
						width = Integer.parseInt(tok.nextToken());
						height = Integer.parseInt(tok.nextToken());
						mines = Integer.parseInt(tok.nextToken());
					}
					// default values
					else {
						width = 10;
						height = 10;
						mines = 10;
					}

					MineSweeper mswp;

					mswp = new MineSweeper(width, height, mines, socketComm);
					mswp.play();

					// socketComm.readLine();

				}
				if ("q".equals(command)) {

					onGame = false;

				}
			}

			// close the socket

		} catch (IOException e) {
			System.err
					.println("An error occured accessing the client, execution stops.");
			System.exit(1);
		} finally {
			try {
				socket.close();
				System.out.println("Connection with " + socket.getInetAddress()
						+ ":" + socket.getPort() + " closed");
			} catch (Exception e) {
				System.err.println("Sorry, could not close socket.");
				e.printStackTrace();
				System.exit(1);
			}
		}
	}

}