/*
 * Modified Battleship game (original made by Gabit Kemelov 
 * http://code.google.com/p/some-examples/source/browse/?r=3
 * GPL3 licence)
 */
package hu.mit.bme.ftsrg.gameEnvironment.battleShip;

import java.io.IOException;
import java.util.*;

import hu.mit.bme.ftsrg.gameEnvironment.SocketCommunication;

/**
 * 
 * The user interface of the Battleship game
 */
public class BattleshipUserInterface {
	
	public void run(BattleshipGame game, SocketCommunication socketComm)
			throws IOException {
		socketComm.writeLine("Welcome to Battleship!");
		socketComm.writeLine("===========================");
		socketComm.writeLine();

		boolean cheat = false; // show the ships when true

		// ---------USER INITIALIZATION BLOCK--------
		socketComm.write("Enter your name and press ENTER: ");
		String name = socketComm.readLine();

		if (name.isEmpty())
			name = "Anonymus";

		socketComm.writeLine(name);

		String battleship = "";
		String cruiser = "";
		String destroyer = "";
		String tug = "";

		boolean validBoard = false;
		boolean userExit = false;

		boardBilder: while (!validBoard) {
			socketComm
					.writeLine("======================================================================");
			socketComm
					.writeLine("Enter the start coordinates of the ships -- H: horizontal, V: vertical");
			socketComm
					.writeLine("======================================================================");

			// Read the Battleships' start coordinates and orientation
			while (!(battleship.matches("[0-9],[0-9],[hHvV]"))
					&& !(battleship.equals("*"))) {
				socketComm
						.write("Enter Battleship location and direction: (e.g. 2,2,H):  ( * : random board)");
				battleship = socketComm.readLine();
				if ("q".equals(battleship)) {
					socketComm
							.writeLine("\n You've succesfully exited from Battleship! \n");
					userExit = true;
					break boardBilder;
				}
			}

			if (battleship.equals("*")) {
				// Random placement class call
				destroyer = "*";
				cruiser = "*";
				tug = "*";
			} else {

				// Read the Destroyers' start coordinates and orientation
				while (!(destroyer.matches("[0-9],[0-9],[hHvV]"))
						|| (destroyer == "*")) {
					socketComm
							.write("Enter Destroyer location and direction: (e.g. 2,3,H): ");
					destroyer = socketComm.readLine();
					if ("q".equals(destroyer)) {
						socketComm
								.writeLine("\n You've succesfully exited from Battleship! \n");
						userExit = true;
						break boardBilder;
					}
				}

				// Read the Cruisers' start coordinates and orientation
				while (!(cruiser.matches("[0-9],[0-9],[hHvV]"))
						|| (cruiser == "*")) {
					socketComm
							.write("Enter Cruiser location and direction: (e.g. 2,4,H): ");
					cruiser = socketComm.readLine();
					if ("q".equals(cruiser)) {
						socketComm
								.writeLine("\n You've succesfully exited from Battleship! \n");
						userExit = true;
						break boardBilder;
					}
				}

				// Read the Tugs' start coordinates and orientation
				while (!(tug.matches("[0-9],[0-9],[hHvV]")) || (tug == "*")) {
					socketComm
							.write("Enter Tug location and direction: (e.g. 2,5,H): ");
					tug = socketComm.readLine();
					if ("q".equals(tug)) {
						socketComm
								.writeLine("\n You've succesfully exited from Battleship! \n");
						userExit = true;
						break boardBilder;
					}
				}
			}
			validBoard = game.userInitialization(name, battleship, destroyer,
					cruiser, tug);
			if (!validBoard) {
				socketComm
						.writeLine("\nSorry, "
								+ name
								+ ", you have incorrectly placed a piece on the board, please retry:");
				battleship = "";
				cruiser = "";
				destroyer = "";
				tug = "";
			}
		}
		socketComm.writeLine();
		// ---------END USER INIT BLOCK--------------

		while ((!game.isWon()) && (!userExit)) {
			// print the game
			game.printGameBoard(cheat);
			socketComm.write("Please Enter Target Coordinates (e.g. 4,3): ");
			String move = socketComm.readLine();

			// Exiting
			if ("q".equals(move)) {
				socketComm
						.writeLine("\n You've succesfully exited from Battleship! \n \n");
				userExit = true;
				break;
			}

			int movex, movey;
			movex = -2;
			movey = -2;

			StringTokenizer tokenizer = new StringTokenizer(move, ",");
			if (tokenizer.countTokens() == 2) {
				try {
					// Tries to parse the string tokens into integers
					movex = Integer.parseInt(tokenizer.nextToken());
					movey = Integer.parseInt(tokenizer.nextToken());
				} catch (Exception e) {/* Doesn't handle exceptions */
				}
			}

			// Cheat code: displays entire board layout
			if (movex == -1 && movey == -1) {
				cheat = true;
			} else {
				// Parse the move
				cheat = false;
				// Treat X as horizonal and Y as vertical, function doe the opposite
				String result = game.move(movey, movex);
				socketComm.writeLine("\n\r" + name + result + "\n");
			}
		}
	}
}
